#!/usr/bin/env python -e

"""
DOCSTRING IS HERE
"""

from platform import system

user_platform = system()

if user_platform == "Windows":

    from .windows import Mixer

elif user_platform == "Darwin":

    from .darwin import Mixer

elif user_platform == "Linux":

    from .linux import Mixer

else:

    raise Exception(
        "Sorry, your platform isn't supported yet by cpmixer ({})".format(
            user_platform
        )
    )

__metadata_version__ = "2.1"
__name__ = "cpmixer"
__version__ = "0.8.0"
__summary__ = "Cross-platform module for controlling audio mixers"
__description__ = "Cross-platform module for controlling audio mixers"
__description_content_type__ = "text/markdown"
__keywords__ = (
    "media workflow volume sound modular cross-platform"
)  # A list of additional keywords to be used to assist searching for the distribution in a larger catalog.
__homepage__ = "https://github.com/drtexx/{}".format(__name__)
__author__ = "Denver Pallis"
__author_email__ = "DenverPallisProjects@gmail.com"
# __maintainer__ = "Denver Pallis" # should be omitted if it is identical to Author
# __maintainer_email__ = "DenverPallisProjects@gmail.com" # should be omitted if it is identical to Author
__license__ = "GPLv3+"
__classifiers__ = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Environment :: MacOS X",
    "Environment :: Win32 (MS Windows)",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    "Natural Language :: English",
    "Operating System :: MacOS",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: POSIX :: Linux",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Topic :: Multimedia",
    "Topic :: Scientific/Engineering :: Human Machine Interfaces",
    "Topic :: Software Development",
    "Topic :: Software Development :: Libraries",
]
__requires_dist__ = ["pyalsaaudio"]  # requirements
__setup_requires__ = ["wheel", "setuptools"]
__requires_python__ = [">=3"]  # required python version
__requires_external__ = []  # external requirements
__project_urls__ = {  # a browsable URL for the project and a label for it, separated by a comma
    "Bug Reports": "https://github.com/drtexx/{}/issues".format(__name__),
    "Source": "https://github.com/drtexx/{}".format(__name__),
    "Funding": "https://paypal.me/denverpallis",
    "Docs": "https://{}.readthedocs.io".format(__name__),
}
__exclude_packages__ = ["contrib", "docs", "tests"]

# metadata standard: https://packaging.python.org/specifications/core-metadata/



class MixerTemplate:
    def __init__(
        self,
        get_mute_method,
        set_mute_method,
        get_volume_method,
        set_volume_method,
    ):
        self.gmute = get_mute_method
        self.smute = set_mute_method
        self.gvol = get_volume_method
        self.svol = set_volume_method



import alsaaudio
from .template import MixerTemplate


class Mixer(MixerTemplate):
    def __init__(self, *args, **kwargs):
        super().__init__(
            get_mute_method=self._gmute,
            set_mute_method=self._smute,
            get_volume_method=self._gvol,
            set_volume_method=self._svol,
            *args,
            **kwargs,
        )
        self.alsa_mixer = alsaaudio.Mixer(control="Master")

    def _gmute(self):

        mute_channels = self.alsa_mixer.getmute()

        mute = self._channel_average(mute_channels)

        return mute

    def _smute(self, new_mute):

        new_mute = bool(new_mute)

        self.alsa_mixer.setmute(new_mute)
        return new_mute

    def _gvol(self):

        volume_channels = self.alsa_mixer.getvolume()

        vol = self._channel_average(volume_channels)

        return vol

    def _svol(self, new_vol):

        new_vol = int(new_vol)

        self.alsa_mixer.setvolume(new_vol)
        return new_vol

    def _channel_average(self, channels):

        channel_sum = 0
        num_channels = len(channels)

        for channel in channels:
            channel_sum += channel

        return int(channel_sum / num_channels)


# print(alsaaudio.mixers())
# mixer = alsaaudio.Mixer(control='Master')
#
# print(mixer.cardname())
# print(mixer.mixer())
# print(mixer.mixerid())
# print(mixer.switchcap())
# print(mixer.volumecap())
# print(mixer.getenum())
# print(mixer.getmute())
# print(mixer.getrange())
# # print(mixer.getrec())
# print(mixer.getvolume())
# # print(mixer.setvolume())
# # print(mixer.setmute())
# # print(mixer.setrec())
# print(mixer.polldescriptors())
# print(mixer.handleevents())
